from typing import List

import pytest
from vpic.models import Model
from vpic.typed_client import TypedClient


class TestModels:
    def test_get_models_for_makeid(self, typed_client: TypedClient, responses):
        models: List[Model] = typed_client.get_models_for_make(441)
        assert len(models) == 5

    def test_get_models_for_makeid_year(self, typed_client: TypedClient, responses):
        models: List[Model] = typed_client.get_models_for_make(441, model_year=2020)
        assert len(models) == 4

    def test_get_models_for_makeid_vehicletype(
        self, typed_client: TypedClient, responses
    ):
        models: List[Model] = typed_client.get_models_for_make(441, vehicle_type="Car")
        assert len(models) == 5

    def test_get_models_for_makeid_year_vehicletype(
        self, typed_client: TypedClient, responses
    ):
        models: List[Model] = typed_client.get_models_for_make(
            441, model_year=2021, vehicle_type="Car"
        )
        assert len(models) == 4

    def test_get_models_for_make(self, typed_client: TypedClient, responses):
        models: List[Model] = typed_client.get_models_for_make("TESLA")
        assert len(models) == 5

    def test_get_models_for_make_year(self, typed_client: TypedClient, responses):
        models: List[Model] = typed_client.get_models_for_make("TESLA", model_year=2020)
        assert len(models) == 4

    def test_get_models_for_make_vehicletype(
        self, typed_client: TypedClient, responses
    ):
        models: List[Model] = typed_client.get_models_for_make(
            "TESLA", vehicle_type="Car"
        )
        assert len(models) == 5

    def test_get_models_for_make_year_vehicletype(
        self, typed_client: TypedClient, responses
    ):
        models: List[Model] = typed_client.get_models_for_make(
            "TESLA", model_year=2020, vehicle_type="Car"
        )
        assert len(models) == 4

    @pytest.mark.xfail()
    def test_get_canadian_vehicle_specifications(
        self, typed_client: TypedClient, responses
    ):
        models: List[Model] = typed_client.get_canadian_vehicle_specifications(
            2011, make="Acura", model="", units=""
        )
        assert len(models) == 10



from logging import exception
from typing import Optional
from requests.models import Response


def handle_error_response(resp: Response):

    HTTP_ERRORS = {
        400: InvalidRequest,
        404: MethodNotFound,
        429: TooManyRequests,
        500: InternalError,
        503: ServiceUnavailable,
    }

    if resp.headers.get('content-type', 'application/json') == 'application/json':
        error = resp.json()
        message = error.get("message")
        detail = error.get("messageDetail")
    else:
        error = ""
        message = ""
        detail = ""

    exc = HTTP_ERRORS.get(resp.status_code, VPICError)
    if detail.startswith("The parameters dictionary"):
        exc = InvalidParameters

    raise exc(message=message, detail=detail, response=resp)



class VPICError(Exception):
    """Base class for vPIC API client exceptions.

    Attributes:
        message : Human readable string describing the exception.
        detail: A more detailed or specific description of the exception.
        response: The ``requests.models.Response`` from the vPIC API.

    """

    message: str = "An unknown error occurred"
    detail: Optional[str] = None
    response: Optional[Response] = None

    def __init__(
        self,
        message: Optional[str] = None,
        detail: Optional[str] = None,
        response: Optional[Response] = None,
    ):
        self.response = response
        if message:
            self.message = message
        if detail:
            self.detail = detail

    def __str__(self):
        return self.message


class InvalidRequest(VPICError):
    pass


class MethodNotFound(VPICError):
    """Method not found in vPIC API."""

    pass


class InvalidParameters(VPICError):
    """You passed an invalid parameter value."""

    pass


class TooManyRequests(VPICError):
    """You made too many requests

    See the Retry-After header for advice about how long to wait
    before submitting another request.

    """

    pass


class InternalError(VPICError):
    """An error occurred in the service"""

    pass


class ServiceUnavailable(VPICError):
    """The service is not available, and may be down for maintenance."""

    pass


class ParseError(VPICError):
    pass



from . import expected_result


class TestManufacturers:
    def test_get_wmis_for_manufacturerid(self, vpic, responses, datadir):
        actual = vpic.get_wmis_for_manufacturer(988)
        expected = expected_result(datadir / "get-wmis-for-manufacturerid.json")
        assert actual == expected

    def test_get_wmis_for_manufacturer_name(self, vpic, responses, datadir):
        actual = vpic.get_wmis_for_manufacturer("Honda")
        expected = expected_result(datadir / "get-wmis-for-manufacturer-name.json")
        assert actual == expected

    def test_get_wmis_for_manufacturer_partialname(self, vpic, responses, datadir):
        actual = vpic.get_wmis_for_manufacturer("TECHNOLOG")
        expected = expected_result(
            datadir / "get-wmis-for-manufacturer-partialname.json"
        )
        assert actual == expected

    def test_get_wmis_for_manufacturer_vehicletype(self, vpic, responses, datadir):
        actual = vpic.get_wmis_for_manufacturer(vehicle_type='mpv')
        expected = expected_result(
            datadir / "get-wmis-for-manufacturer-vehicletype.json"
        )
        assert actual == expected

    def test_get_all_manufacturers(self, vpic, responses, datadir):
        assert vpic.get_all_manufacturers(
            "Completed Vehicle Manufacturer", 1
        ) == expected_result(datadir / "get-all-manufacturers.json")

    def test_get_manufacturer_details(self, vpic, responses, datadir):
        assert vpic.get_manufacturer_details(988) == expected_result(
            datadir / "get-manufacturer-details.json"
        )

    def test_get_manufacturer_details_name(self, vpic, responses, datadir):
        assert vpic.get_manufacturer_details("Honda") == expected_result(
            datadir / "get-manufacturer-details-name.json"
        )

    def test_get_manufacturer_details_by_partial_name(self, vpic, responses, datadir):
        assert vpic.get_manufacturer_details("METRO") == expected_result(
            datadir / "get-manufacturer-details-partialname.json"
        )

    def test_get_equipment_plant_codes(self, vpic, responses, datadir):
        assert vpic.get_equipment_plant_codes(2016, 1) == expected_result(
            datadir / "get-equipment-plant-codes.json"
        )

    def test_get_parts_565(self, vpic, responses, datadir):
        assert vpic.get_parts("565", "2015-01-01", "2015-05-05", 1) == expected_result(
            datadir / "get-parts-565.json"
        )

    def test_get_parts_566(self, vpic, responses, datadir):
        assert vpic.get_parts("566", "2016-02-01", "2016-02-15", 1) == expected_result(
            datadir / "get-parts-566.json"
        )



import json

import pytest
from requests import Response

from vpic.exceptions import (
    InvalidParameters,
    MethodNotFound,
    handle_error_response,
)


def _response(content, status_code):
    r = Response()
    r.status_code = status_code
    r._content = json.dumps(content).encode()
    return r


def test_invalid_parameter():
    r = _response(
        {
            "message": "The request is invalid.",
            "messageDetail": "The parameters dictionary contains a null entry \
                for parameter 'id' of non-nullable type 'System.Int32' for \
                    method 'System.Net.Http.HttpResponseMessage \
                        GetVehicleTypesForMakeId(Int32, System.String)' in \
                            'APIEF.Controllers.VehiclesController'. An optional \
                                parameter must be a reference type, a nullable type, \
                                    or be declared as an optional parameter.",
        },
        status_code=400,
    )

    with pytest.raises(InvalidParameters) as exc_info:
        handle_error_response(r)

    e = exc_info.value
    assert e.response == r


def test_missing_method():
    r = _response(
        {
            "message": "No HTTP resource was found that matches the \
                request URI 'https://app-prod-vpic-api.nhtsa-prod-ext.\
                    appserviceenvironment.net/api/vehicles/DecodeVinValues/\
                        ?modelyear=1891&format=json'.",
            "messageDetail": "No action was found on the controller \
                'Vehicles' that matches the request.",
        },
        status_code=404,
    )

    with pytest.raises(MethodNotFound) as exc_info:
        handle_error_response(r)

    e = exc_info.value
    assert e.response == r



from . import expected_result


class TestVariables:
    def test_get_vehicle_variable_list(self, vpic, responses, datadir):
        assert vpic.get_vehicle_variable_list() == expected_result(
            datadir / "get-vehicle-variable-list.json"
        )

    def test_get_vehicle_variable_values_list(self, vpic, responses, datadir):
        assert vpic.get_vehicle_variable_values_list("Vehicle Type") == expected_result(
            datadir / "get-vehicle-variable-values-list.json"
        )



import pytest
from requests import Request

from vpic.session import VpicAPISession


@pytest.fixture
def fake_request():
    return Request("GET", "http://someurl/")


def test_request_headers(fake_request):
    s = VpicAPISession()
    r = s.prepare_request(fake_request)

    assert r.headers["Accept-Charset"] == "utf-8"
    # assert r.headers["Content-Type"] == "text/plain"



import json
from pathlib import Path

import pytest
import responses as responses_

from vpic import Client


@pytest.fixture(scope="session")
def vpic() -> Client:
    return Client()


@pytest.fixture(scope="session")
def responses():

    endpoints = {}

    def _request_callback():
        def _handle(request):
            response_file = endpoints[request.url]["response_file"]
            with Path(response_file).open(encoding='utf-8') as fp:
                resp = json.load(fp)
            return (
                resp["status"],
                {},
                json.dumps(resp["content"]),
            )

        return _handle

    def _response_callback(resp):
        # placeholder
        resp.callback_processed = True
        return resp

    with responses_.RequestsMock(
        assert_all_requests_are_fired=False, response_callback=_response_callback
    ) as requests_mock:

        response_files = list(Path("tests").glob("**/*.json"))
        for rf in response_files:
            with rf.open(encoding='utf-8') as fp:
                resp = json.load(fp)
                endpoints[resp["url"]] = {
                    "method": resp.get("method", "GET"),
                    "status": resp["status"],
                    "response_file": str(rf),
                }

        for url, spec in endpoints.items():
            requests_mock.add_callback(
                spec["method"],
                url,
                callback=_request_callback(),
            )
        yield requests_mock



import re
from typing import Dict, List, Any, Union

_STANDARD_VARIABLE_NAMES: Dict[str, str] = {
    "ID": "Id",
    "GCWR": "GCWRFrom",
    "GCWR_to": "GCWRTo",
    "GVWR": "GVWRFrom",
    "GVWR_to": "GVWRTo",
    "Make_ID": "MakeId",
    "MakeID": "MakeId",
    "Make_Name": "Make",
    "MakeName": "Make",
    "Mfr_CommonName": "ManufacturerCommonName",
    "Mfr_ID": "ManufacturerId",
    "Mfr_Name": "Manufacturer",
    "ManufacturerName": "Manufacturer",
    "MfrId": "ManufacturerId",
    "MfrName": "Manufacturer",
    "Model_ID": "ModelId",
    "Model_Name": "Model",
    "ModelName": "Model",
    "ModelID": "ModelId",
    "VehicleTypeName": "VehicleType",
}


def standardize(object: Union[Dict, List[Dict]]):
    """Standardize response variable names.

    vPIC responses sometimes use different names for the same variable,
    so we'll standardize them before returning to the caller.

    Args:
        object: a vPIC response object

    Returns:
        object, with standardized keys

    Example:
        >>> standardize({ 'Mfr_ID' : 232 })
        { 'ManufacturerId' : 232 }

    """
    if isinstance(object, dict):
        return {
            _STANDARD_VARIABLE_NAMES.get(key, key): standardize(value)
            for key, value in object.items()
        }
    elif isinstance(object, list):
        return [standardize(item) for item in object]
    else:
        return object


_VARIABLE_NAME_RE = re.compile(r"([A-Z]+)(?=([a-z_]|))")

_WORDS_TO_SEPARATE: Dict[str, str] = {
    "ncsa": "ncsa_",
    "evdrive": "ev_drive",
    "sae": "sae_",
    "dotcode": "dot_code",
    "gcwrfrom": "gcwr_from",
    "gvwrfrom": "gvwr_from",
    "gcwrto": "gcwr_to",
    "gvwrto": "gvwr_to",
}


def snake_case(object: Dict[str, Any]) -> Dict[str, Any]:
    """Convert response variable names (JSON keys) to snake case.

    TypedClient returns model objects with pythonic property names,
    so we need to transform variable names in vPIC responses to
    snake case.

    Args:
        object: a vPIC response object

    Returns:
        A new object with snake-cased variable names (keys)

    Example:
        >>> snake_case({ 'ManufacturerId' : 232 })
        { 'manufacturer_id' : 232 }

    """
    new_object: Dict[str, Any] = {}
    for key, value in object.items():
        new_key = _VARIABLE_NAME_RE.sub(r"_\1", key).lower()
        new_key = new_key.replace("__", "_")
        if new_key[0] == "_":
            new_key = new_key[1:]

        for compound_word, separated_word in _WORDS_TO_SEPARATE.items():
            if compound_word in new_key:
                new_key = new_key.replace(compound_word, separated_word)
                break

        if isinstance(value, list):
            value = [snake_case(v) for v in value]

        new_object[new_key] = value
    return new_object



from typing import List
from vpic.models import Document, ManufacturerDetail, PlantCode, WMI
from vpic.typed_client import TypedClient


class TestManufacturers:
    def test_get_wmis_for_manufacturerid(self, typed_client: TypedClient, responses):
        wmis: List[WMI] = typed_client.get_wmis_for_manufacturer(988)
        assert len(wmis) == 20

    def test_get_wmis_for_manufacturer_name(self, typed_client: TypedClient, responses):
        wmis: List[WMI] = typed_client.get_wmis_for_manufacturer(
            "Honda"
        )
        assert len(wmis) == 44

    def test_get_wmis_for_manufacturer_partialname(
        self, typed_client: TypedClient, responses
    ):
        wmis: List[WMI] = typed_client.get_wmis_for_manufacturer(
            "TECHNOLOG"
        )
        assert len(wmis) == 101

    def test_get_manufacturer_details(self, typed_client: TypedClient, responses):
        details: List[ManufacturerDetail] = typed_client.get_manufacturer_details(988)
        assert len(details) == 1
        honda = details[0]
        assert honda.manufacturer_id == 988
        assert (
            honda.manufacturer
            == "HONDA DEVELOPMENT & MANUFACTURING OF AMERICA, LLC"
        )
        assert honda.manufacturer_common_name == "Honda"
        assert honda.manufacturer_types is not None
        if honda.manufacturer_types is not None:
            assert honda.manufacturer_types[0].name == "Completed Vehicle Manufacturer"

    def test_get_manufacturer_details_name(self, typed_client: TypedClient, responses):
        details: List[ManufacturerDetail] = typed_client.get_manufacturer_details(
            "Honda"
        )

        assert details[0].manufacturer_id == 987
        assert details[0].manufacturer == "HONDA MOTOR CO., LTD"
        assert details[0].manufacturer_common_name == "Honda"

    def test_get_manufacturer_details_by_partial_name(
        self, typed_client: TypedClient, responses
    ):
        details: List[ManufacturerDetail] = typed_client.get_manufacturer_details(
            "METRO"
        )
        assert details[0].manufacturer_id == 3382
        assert details[0].manufacturer == "NICHIBEI METROPAGES, INC."
        assert details[0].manufacturer_common_name is None

    def test_get_equipment_plant_codes(self, typed_client: TypedClient, responses):
        plant_codes: List[PlantCode] = typed_client.get_equipment_plant_codes(2016, 1)
        assert len(plant_codes) == 1091
        assert hasattr(plant_codes[0], "dot_code")

    def test_get_parts_565(self, typed_client: TypedClient, responses):
        documents: List[Document] = typed_client.get_parts(
            "565", "2015-01-01", "2015-05-05", 1
        )
        assert len(documents) == 175

    def test_get_parts_566(self, typed_client: TypedClient, responses):
        documents: List[Document] = typed_client.get_parts(
            "566", "2016-02-01", "2016-02-15", 1
        )
        assert len(documents) == 15



from typing import List

from vpic.models import Make, VehicleType
from vpic.typed_client import TypedClient


class TestMakes:
    def test_get_all_makes(self, typed_client: TypedClient, responses):
        makes: List[Make] = typed_client.get_all_makes()
        assert len(makes) == 9628
        assert makes[0].make == "ASTON MARTIN"
        assert makes[0].make_id == 440
        assert makes[1].make == "TESLA"
        assert makes[1].make_id == 441

    def test_get_makes_for_manufacturer(self, typed_client: TypedClient, responses):
        makes: List[Make] = typed_client.get_makes_for_manufacturer(988)
        assert len(makes) == 2

    def test_get_makes_for_manufacturer_by_name(
        self, typed_client: TypedClient, responses
    ):
        makes: List[Make] = typed_client.get_makes_for_manufacturer("Honda")
        assert len(makes) == 15

    def test_get_makes_for_manufacturer_by_partial_name(
        self, typed_client: TypedClient, responses
    ):
        makes: List[Make] = typed_client.get_makes_for_manufacturer("METRO")
        assert len(makes) == 4

    def test_get_makes_for_manufacturerid_and_modelyear(
        self, typed_client: TypedClient, responses
    ):
        makes: List[Make] = typed_client.get_makes_for_manufacturer(988, 2021)
        assert len(makes) == 2

    def test_get_makes_for_vehicle_type(self, typed_client: TypedClient, responses):
        makes: List[Make] = typed_client.get_makes_for_vehicle_type("Car")
        assert len(makes) == 167

    def test_get_vehicle_types_for_make_id(self, typed_client: TypedClient, responses):
        types: List[VehicleType] = typed_client.get_vehicle_types_for_make(474)
        assert len(types) == 5

    def test_get_vehicle_types_for_make_name(
        self, typed_client: TypedClient, responses
    ):
        types: List[VehicleType] = typed_client.get_vehicle_types_for_make("kia")
        assert len(types) == 3



import pytest
from vpic.exceptions import MethodNotFound

from . import expected_result


class TestMakes:
    def test_get_all_makes(self, vpic, responses, datadir):
        assert vpic.get_all_makes() == expected_result(datadir / "get-all-makes.json")

    def test_get_makes_for_manufacturer(self, vpic, responses, datadir):
        assert vpic.get_makes_for_manufacturer(988) == expected_result(
            datadir / "get-makes-for-manufacturer.json"
        )

    def test_get_makes_for_manufacturer_by_name(self, vpic, responses, datadir):
        assert vpic.get_makes_for_manufacturer("Honda") == expected_result(
            datadir / "get-makes-for-manufacturer-name.json"
        )

    def test_get_makes_for_manufacturer_by_partial_name(self, vpic, responses, datadir):
        assert vpic.get_makes_for_manufacturer("METRO") == expected_result(
            datadir / "get-makes-for-manufacturer-partial-name.json"
        )

    def test_get_makes_for_manufacturerid_and_modelyear(self, vpic, responses, datadir):
        assert vpic.get_makes_for_manufacturer(988, 2021) == expected_result(
            datadir / "get-makes-for-manufacturerid-and-modelyear.json"
        )

    def test_get_makes_for_vehicle_type(self, vpic, responses, datadir):
        assert vpic.get_makes_for_vehicle_type("Car") == expected_result(
            datadir / "get-makes-for-vehicletype.json"
        )

    # def test_get_makes_for_vehicle_type(self, vpic):
    #     """VPIC always returns 'Truck ' for VehicleType 3, but doesn't accept that in this API call"""
    #     responses.add_passthru('https://...')
    #     with pytest.raises(MethodNotFound):
    #         vpic.get_makes_for_vehicle_type("Truck ")

    def test_get_vehicle_types_for_make_id(self, vpic, responses, datadir):
        assert vpic.get_vehicle_types_for_make(474) == expected_result(
            datadir / "get-vehicletypes-for-makeid.json"
        )

    def test_get_vehicle_types_for_make_name(self, vpic, responses, datadir):
        assert vpic.get_vehicle_types_for_make("kia") == expected_result(
            datadir / "get-vehicletypes-for-makename.json"
        )



from . import expected_result


class TestDecode:
    def test_decode_vin(self, vpic, responses, datadir):
        assert vpic.decode_vin("1FTMW1T88MFA00001") == expected_result(
            datadir / "decode-vin.json"
        )

    def test_decode_vin_with_modelyear(self, vpic, responses, datadir):
        assert vpic.decode_vin("1FTMW1T88MFA00001", 2021) == expected_result(
            datadir / "decode-vin-with-modelyear.json"
        )

    def test_decode_vin_extended(self, vpic, responses, datadir):
        assert vpic.decode_vin(
            "1FTMW1T88MFA00001", 2021, extend=True
        ) == expected_result(datadir / "decode-vin-extended.json")

    def test_decode_vin_unflattened(self, vpic, responses, datadir):
        assert vpic.decode_vin(
            "1FTMW1T88MFA00001", 2021, flatten=False
        ) == expected_result(datadir / "decode-vin-unflattened.json")

    def test_decode_partial_vin(self, vpic, responses, datadir):
        assert vpic.decode_vin("5UXWX7C5*BA", 2011) == expected_result(
            datadir / "decode-partial-vin.json"
        )

    def test_decode_partial_vin_unflattened(self, vpic, responses, datadir):
        assert vpic.decode_vin(
            "1FA6P8C0!M!100001", 2021, flatten=False
        ) == expected_result(datadir / "decode-partial-vin-unflattened.json")

    def test_decode_vin_batch(self, vpic, responses, datadir):
        assert vpic.decode_vin_batch(
            ["5UXWX7C5*BA,2011", "5YJSA3DS*EF"]
        ) == expected_result(datadir / "decode-vin-batch.json")

    def test_decode_wmi(self, vpic, datadir, responses):
        actual = vpic.decode_wmi("1FT")
        expected = expected_result(datadir / "decode-wmi.json")
        assert actual == expected



import logging
from typing import List, Optional, Union

import desert
import marshmallow

from .client import Client
from .models import (
    Document,
    Make,
    Manufacturer,
    ManufacturerDetail,
    Model,
    PlantCode,
    Value,
    Variable,
    Vehicle,
    VehicleType,
    WMI,
)
from .transforms import snake_case

log = logging.getLogger(__name__)


class TypedClient:
    """A client library for the U.S. NHTSA vPIC API

    ``TypedClient`` returns model objects instead of JSON responses. If you
    need the original vPIC JSON responses, use ``vpic.Client``.

    A client library for the United States National Highway Traffic Safety
    Administration (NHTSA) Vehicle Product Information Catalog (vPIC) Vehicle
    Listing API.

    Use this to gather information on vehicles and their specifications,
    and to decode VINs to extract information for specific vehicles. vPIC
    has information about these types of vehicles sold or imported in
    the USA:

    * Bus
    * Incomplete Vehicle
    * Low Speed Vehicle (LSV)
    * Motorcycle
    * Multipurpose Passenger Vehicle (MPV)
    * Passenger Car
    * Trailer
    * Truck

    vPIC has all of the information about how manufacturers assign a VIN that
    encodes the vehicles characteristics. Vehicle manufacturers provide this
    information to NHTSA under U.S. law 49 CFR Part 565.

    The API available 24/7, is free to use, and does not require registration.
    NHTSA uses automatic traffic rate controls to maintain the performance of
    the API and their websites that use the API.

    NHTSA occasionally adds, removes, or renames variables in vPIC API
    responses. This class uses Marshmallow to deserialize the response.
    When a variable is missing or renamed, methods in this class raise
    ``marshmallow.exceptions.ValidationError``. If you instantiate this class
    with unknown='RAISE', methods will also raise
    ``marshmallow.exceptions.ValidationError`` when the vPIC response
    includes a new variable that isn't defined in the model (models.py).

    See https://vpic.nhtsa.dot.gov/api for more on the API.

    Attributes:
        host: Hostname, including http(s)://, of the vPIC instance to query.
        unknowns: exclude new API response variables ('EXCLUDE'), or raise an
            exception ('RAISE') instead

    """

    def __init__(
        self,
        host: Optional[str] = "https://vpic.nhtsa.dot.gov/api/vehicles/",
        unknown: Optional[str] = "EXCLUDE",
    ):
        if unknown == "EXCLUDE":
            self._meta = {"unknown": marshmallow.EXCLUDE}
        elif unknown == "RAISE":
            self._meta = {"unknown": marshmallow.RAISE}
        else:
            raise ValueError("unknown must be 'EXCLUDE' or 'RAISE'")
        self._client = Client(host, standardize_variables=True)

    def decode_vin(
        self, vin: str, model_year: Optional[int] = None, extend: Optional[bool] = False
    ) -> Vehicle:
        """Decode a 17-digit Vehicle Identification Number (VIN) or partial VIN.

        Decode the make, model, series, trim, and other vehicle information
        from VIN. Model year is required for pre-1980 vehicles, though vPIC
        recommends that you always pass it.

        If you don't have a complete 17-digit VIN, you can pass a partial
        VIN, using asterisk (*) for missing characters. The VIN check digit
        (the 9th character) isn't required for partial VINs. The shorter the
        partial VIN, the less vehicle information you'll receive in the
        response.

        See get_vehicle_variable_list for the variables returned here.

        Args:
            vin: A 17-digit VIN or partial VIN. Use asterisk for missing
                characters.
            model_year: The vehicle's model year. Recommended, but not required.
            extend: If True, response will include variables for other NHTSA
                programs like NCSA. Defaults to False.

        Returns:
            A ``Vehicle`` with the information decoded from ``vin``.

        Raises:
            ValueError: if ``vin`` is missing or isn't 6 to 17 characters long.
            ValueError: if ``model_year`` is earlier than 1981.

        Example:
            >>> decode_vin('1FTMW1T88MFA00001')
            Vehicle(
                abs="",
                ...
                body_cab_type="Crew/ Super Crew/ Crew Max",
                body_class="Pickup",
                brake_system_desc="",
                brake_system_type="Hydraulic",
                ...
                displacement_cc="3500.0",
                displacement_ci="213.58310433156",
                displacement_l="3.5",
                ...
                drive_type="4WD/4-Wheel Drive/4x4",
                ...
                engine_configuration="V-Shaped",
                engine_cycles="",
                engine_cylinders="6",
                engine_hp="375",
                engine_hp_to="",
                engine_kw="279.6375",
                engine_manufacturer="Ford",
                engine_model="GTDI",
                entertainment_system="",
                error_code="0",
                error_text="0 - VIN decoded clean. Check Digit (9th position) is co...",
                ...
                make="FORD",
                make_id="460",
                manufacturer="FORD MOTOR COMPANY, USA",
                manufacturer_id="976",
                model="F-150",
                model_id="1801",
                model_year="2021",
                motorcycle_chassis_type="Not Applicable",
                motorcycle_suspension_type="Not Applicable",
                ...
                plant_city="DEARBORN",
                plant_company_name="",
                plant_country="UNITED STATES (USA)",
                plant_state="MICHIGAN",
                ...
                series="F-Series",
                series2="",
                ...
                trim="SuperCrew-SSV",
                ...
                vin="1FTMW1T88MFA00001",
                ...
                vehicle_type="TRUCK ",
                ...
            )

        """
        schema = desert.schema(Vehicle, meta=self._meta)
        return schema.load(snake_case(self._client.decode_vin(vin, model_year, extend)))

    def decode_vin_batch(self, vins: List[str]) -> List[Vehicle]:
        """Decode a batch of 17-digit VINs or partial VINs.

        Model year is required for pre-1980 vehicles, though vPIC recommends
        that you always pass it.

        If you don't have a complete 17-digit VIN, you can pass a partial
        VIN, using asterisk (*) for missing characters. The VIN check digit
        (the 9th character) isn't required for partial VINs. The shorter the
        partial VIN, the less vehicle information you'll receive in the
        response.

        Vehicle variables will be returned in key-value pairs, the same
        format returned by decode_vin(.., flatten=True).

        See get_vehicle_variable_list for the variables returned here.

        Args:
            vins: A list of 17-digit VIN or partial VINs and optional model year.
                Use asterisk for missing characters. For example: ["VIN, model_year",
                "VIN, model_year", ...]

        Returns:
            A list of ``Vehicle``s with the information decoded from ``vins``.

        Raises:
            ValueError: if ``vin`` is missing or isn't 6 to 17 characters long.
            ValueError: if ``model_year`` is earlier than 1981.

        Example:
            >>> decode_vin_batch(["5UXWX7C5*BA,2011", "5YJSA3DS*EF"])
            [Vehicle(...), Vehicle(...)]

        """
        vehicles = self._client.decode_vin_batch(vins)
        schema = desert.schema(Vehicle, meta=self._meta)
        return [schema.load(snake_case(v)) for v in vehicles]

    def decode_wmi(self, wmi: str) -> WMI:
        """Decode a WMI to get manufacturer information

        Provides information on the World Manufacturer Identifier for a
        specific WMI code.

        Args:
            wmi: A 3-character or 6-character World Manufacturer Index code.
                 Large volume manufacturers usually have a  3 character WMI
                 representing positions 1 to 3 ("JTD") of a VIN. Smaller
                 manufacturers have a 6 character WMI representing positions
                 1 to 3 and 12 to 14 of a VIN.

        Returns:
            A ``WorldManufacturerIndex`` with information about the manufacturer.

        Raises:
            ValueError: if ``wmi`` is missing or isn't 3 or 6 characters long.

        Example:
            >>> decode_wmi('1FT')
            WorldManufacturerIndex(
                common_name="Ford",
                created_on="2015-03-23",
                date_available_to_public="2015-01-01",
                make="FORD",
                manufacturer="FORD MOTOR COMPANY, USA",
                parent_company_name="",
                updated_on=None,
                url="http://www.ford.com/",
                vehicle_type="Truck ",
            )

        """
        schema = desert.schema(WMI, meta=self._meta)
        return schema.load(snake_case(self._client.decode_wmi(wmi)))

    def get_wmis_for_manufacturer(
        self,
        manufacturer: Union[str, int],
        vehicle_type: Optional[Union[str, int]] = None,
    ) -> List[WMI]:
        """Returns the WMIs for one or all manufacturers

        Args:
            manufacturer: Pass the Manufacturer Id (int) or the complete
                manufacturer name (str) to return WMIs for a single manufacturer.
                Pass a partial name to return WMIs for all manufacturers with
                names that include the partial name.
            vehicle_type: Pass the vehicle_type Id (int) or complete vehicle_type
                name to return WMIs for that vehicle_type. Pass a partial name to
                return WMIs for vehicle_types matching that name.

        Returns:
            A list of ``WorldManufacturerIndex`` with information about the
            manufacturers.

        Raises:
            ValueError: if ``manufacturer`` is missing

        Examples:
            >>> get_wmis_for_manufacturer('Honda')
            [
                WorldManufacturerIndex(
                    created_on='2015-03-26',
                    date_available_to_public='2015-01-01',
                    manufacturer='HONDA MOTOR CO., LTD',
                    updated_on='2015-06-04',
                    vehicle_type='Passenger Car',
                    wmi='JHM', common_name='',
                    country=None,
                    make='',
                    manufacturer_id=987,
                    parent_company_name='',
                    url=''
                ),
                ...
            ]

        """
        wmis = self._client.get_wmis_for_manufacturer(manufacturer, vehicle_type)
        schema = desert.schema(WMI, meta=self._meta)
        return [schema.load(snake_case(wmi)) for wmi in wmis]

    def get_all_makes(self) -> List[Make]:
        """Returns all of the makes registered with vPIC.

        Returns:
            A list of all of the ``Make``s registered with vPIC.

        Examples:
            >>> get_all_makes()
            [
                Make(make_id=440, make='ASTON MARTIN'),
                Make(make_id=441, make='TESLA'),
                Make(make_id=442, make='JAGUAR'),
                Make(make_id=443, make='MASERATI'),
                Make(make_id=444, make='LAND ROVER'),
                Make(make_id=445, make='ROLLS ROYCE'),
                Make(make_id=446, make='BUELL (EBR)'),
                Make(make_id=447, make='JIALING'),
                Make(make_id=448, make='TOYOTA'),
                Make(make_id=449, make='MERCEDES-BENZ'),
                ...
            ]

        """
        makes = self._client.get_all_makes()
        schema = desert.schema(Make, meta=self._meta)
        return [schema.load(snake_case(make)) for make in makes]

    def get_parts(
        self, cfr_part: str, from_date: str, to_date: str, page: int = 1
    ) -> List[Document]:
        """Returns a list of vehicle documentation submitted by manufacturers.

        Manufacturers provide vehicle information to NHTSA to comply with these
        regulations:

        * 49 CFR Part 565 (Vehicle Identification Number Guidance)
        * 49 CFR Part 566 (Manufacturer Identification – Reporting Requirements)

        This provides a list of documents submitted in a date range. Up to 1,000
        results will be returned at a time.

        Args:
            cfr_part: '565' to return 49 CFR Part 565 submissions;
                '566' to return 49 CFR Part 566 submissions
            from_date: the beginning of the date range to search
            to_date: the end of the date range to search
            page: results are paginated; this is page number to return

        Returns:
            A list of ``Document``s registered with vPIC.

        Raises:
            ValueError: if ``cfr_part`` is missing

        Examples:
            >>> get_parts('565', '2015-01-01', '2015-05-05', 1)
            [
                Document(
                    cover_letter_url='',
                    letter_date='1/1/2015',
                    manufacturer_id=959,
                    manufacturer='MASERATI NORTH AMERICA, INC.',
                    name='ORG13044',
                    url='...',
                    type=None,
                    model_year_from=20,
                    model_year_to=None
                ),
                ...
            ]

        """
        documents = self._client.get_parts(cfr_part, from_date, to_date, page)
        for doc in documents:
            # snake_case doesn't handle lowerUPPER
            doc["CoverLetterUrl"] = doc["CoverLetterURL"]
            del doc["CoverLetterURL"]
        schema = desert.schema(Document, meta=self._meta)
        return [schema.load(snake_case(doc)) for doc in documents]

    def get_all_manufacturers(
        self, manufacturer_type: Optional[str] = None, page: int = 1
    ) -> List[Manufacturer]:
        """Return a list of vPIC manufacturers of the given manufacturer_type.

        This provides a list of all the Manufacturers available in vPIC Dataset.

        See ``get_vehicle_variable_values_list("Manufacturer Type")`` for the list
        of manufacturer types.

        Args:
            manufacturer_type: The manufacturer type, which is Incomplete Vehicles,
                Completed Vehicle Manufacturer, Incomplete Vehicle Manufacturer,
                Intermediate Manufacturer, Final-Stage Manufacturer, Alterer,
                Replica Vehicle Manufacturer. You can pass the full type name, or a
                substring of the type.
            page: results are paginated; this is the page number to return

        Returns:
            A list of the ``Manufacturers``s registered with vPIC.

        Examples:
            >>> get_all_manufacturers("Completed Vehicle", 1)
            [
                {
                "Country": "UNITED STATES (USA)",
                "Mfr_CommonName": "Tesla",
                "Mfr_ID": 955,
                "Mfr_Name": "TESLA, INC.",
                "VehicleTypes": [
                    {
                        "IsPrimary": true,
                        "Name": "Passenger Car"
                    },
                    {
                        "IsPrimary": false,
                        "Name": "Multipurpose Passenger Vehicle (MPV)"
                    }
                },
                ...
            ]

        """
        raise NotImplementedError

    def get_manufacturer_details(
        self, manufacturer: Union[str, int]
    ) -> List[ManufacturerDetail]:
        """Returns details for one or more manufacturers.

        Args:
            manufacturer: Pass the Manufacturer Id (int) or the complete
                manufacturer name (str) to return detail for a single
                manufacturer. Pass a partial name to return manufacturers
                with names that include the partial name.

        Returns:
            A list of ``ManufacturerDetail``s for the manufacturer(s)

        Examples:
            >>> get_manufacturer_details(988)
            [
                ManufacturerDetail(
                    manufacturer_id=988,
                    manufacturer="HONDA DEVELOPMENT & MANUFACTURING OF AMER...",
                    manufacturer_common_name="Honda",
                    address="1919 Torrance Blvd.",
                    address2=None,
                    city="Torrance",
                    contact_email="...@ahm.honda.com",
                    contact_fax=None,
                    contact_phone="...",
                    country="UNITED STATES (USA)",
                    dbas="Marysville Auto Plant and East Liberty Auto Plant; ...",
                    equipment_items=[],
                    last_updated="/Date(1618422117803-0400)/",
                    manufacturer_types=[
                        ManufacturerType(
                            name="Completed Vehicle Manufacturer")
                        ],
                    other_manufacturer_details=None,
                    postal_code="90501",
                    primary_product=None,
                    principal_first_name="...",
                    principal_last_name=None,
                    principal_position="President & CEO",
                    state_province="CALIFORNIA",
                    submitted_name="...",
                    submitted_on="/Date(1618286400000-0400)/",
                    submitted_position="Sr. Specialist, II",
                    vehicle_types=[
                        VehicleType(
                            vehicle_type="Passenger Car",
                            vehicle_type_id=None,
                            make_id=None,
                            make=None,
                            gvwr_from="Class 1A: 3,000 lb or less (1,360 kg or less)",
                            gvwr_to="Class 1D: 5,001 - 6,000 lb (2,268 - 2,722 kg)",
                            is_primary=True
                        ),
                        VehicleType(
                            vehicle_type="Truck ",
                            vehicle_type_id=None,
                            make_id=None,
                            make=None,
                            gvwr_from="Class 2E: 6,001 - 7,000 lb (2,722 - 3,175 kg)",
                            gvwr_to="Class 2E: 6,001 - 7,000 lb (2,722 - 3,175 kg)",
                            is_primary=False
                        ),
                        VehicleType(
                            vehicle_type="Multipurpose Passenger Vehicle (MPV)",
                            vehicle_type_id=None,
                            make_id=None,
                            make=None,
                            gvwr_from="Class 1B: 3,001 - 4,000 lb (1,360 - 1,814 kg)",
                            gvwr_to="Class 2E: 6,001 - 7,000 lb (2,722 - 3,175 kg)",
                            is_primary=False
                        )
                    ]
                )
            ]

        """
        results = self._client.get_manufacturer_details(manufacturer)
        for r in results:
            r["dbas"] = r["DBAs"]
            del r["DBAs"]

            # r["ManufacturerTypes"] = [
            #     ManufacturerType(name=mt["Name"]) for mt in r["ManufacturerTypes"]
            # ]
            # r["VehicleTypes"] = [
            #     VehicleType(
            #         vehicle_type=vt["Name"],
            #         is_primary=vt["IsPrimary"],
            #         gvwr_from=vt["GVWRFrom"],
            #         gvwr_to=vt["GVWRTo"],
            #     )
            #     for vt in r["VehicleTypes"]
            # ]

            # r["ManufacturerTypes"] = [snake_case(mt) for mt in r["ManufacturerTypes"]]
            # r["VehicleTypes"] = [snake_case(vt) for vt in r["VehicleTypes"]]

        schema = desert.schema(ManufacturerDetail, meta=self._meta)
        snake_cased = [snake_case(r) for r in results]
        return [schema.load(sc) for sc in snake_cased]

    def get_makes_for_manufacturer(
        self, manufacturer: Union[str, int], model_year: Optional[int] = None
    ) -> List[Make]:
        """Returns makes produced by a manufacturer or manufacturers.

        Args:
            manufacturer: Pass the Manufacturer Id (int) or the complete
                manufacturer name (str) to return detail for a single manufacturer.
                Pass a partial name to return manufacturers with names that include
                the partial name.
            model_year: Pass a model year to return only those makes made by
                the manufacturer for that model year.

        Returns:
            A list of ``Makes``s produced by the manufacturer(s)

        Raises:
            ValueError: if ``manufacturer`` is missing

        Examples:
            >>> get_makes_for_manufacturer(988)
            [
                Make(
                    make_id:474
                    make:'HONDA'
                    manufacturer_id:None
                    manufacturer:'HONDA DEVELOPMENT & MANUFACTURING OF AMERICA...'
                ),
                Make(
                    make_id=475,
                    make='ACURA',
                    manufacturer_id=None,
                    manufacturer='HONDA DEVELOPMENT & MANUFACTURING OF AMERICA...'
                )
                ...
            ]

        """
        makes = self._client.get_makes_for_manufacturer(manufacturer, model_year)
        schema = desert.schema(Make, meta=self._meta)
        return [schema.load(snake_case(m)) for m in makes]

    def get_makes_for_vehicle_type(self, vehicle_type: str) -> List[Make]:
        """Returns makes that produce a vehicle_type

        Args:
            vehicle_type: A vPIC vehicle_type. For example, "Passenger Car",
                "Truck", or "Multipurpose Passenger Vehicle (MPV)". If you pass
                a partial vehicle_type, for example "Passenger", results will
                include makes for all matching vehicle types. Matching is not
                case sensitive.

        Returns:
            A list of ``Makes``s that produce the ``vehicle_type``

        Raises:
            ValueError: if ``vehicle_type`` is missing

        Examples:
            >>> get_makes_for_vehicle_type('Car')
            [
                Make(
                    make_id=440,
                    make='ASTON MARTIN',
                    manufacturer_id=None,
                    manufacturer=None,
                    vehicle_type_id=2,
                    vehicle_type='Passenger Car'
                )
                ...
            ]

        """
        makes = self._client.get_makes_for_vehicle_type(vehicle_type)
        schema = desert.schema(Make, meta=self._meta)
        return [schema.load(snake_case(m)) for m in makes]

    def get_vehicle_types_for_make(self, make: Union[str, int]) -> List[VehicleType]:
        """Returns vehicle types produced by a make or make

        Args:
            make: Pass the MakeId (int) or the complete make name (str) to return
                vehicle types for a single manufacturer. Pass a partial make name
                to return vehicle types for all makes that match the partial name.
                When you pass a make name, results will include the MakeId and
                MakeName because you may get vehicle_types for more than one make.

        Returns:
            A list of ``VehicleType``s produced by the ``make``(s)

        Raises:
            ValueError: if ``make`` is missing

        Examples:
            >>> get_vehicle_types_for_make(474)
            [
                VehicleType(
                    vehicle_type_id=1,
                    vehicle_type='Motorcycle',
                    make_id=None,
                    make=None
                ),
                VehicleType(
                    vehicle_type_id=2,
                    vehicle_type='Passenger Car',
                    make_id=None,
                    make=None
                ),
                VehicleType(
                    vehicle_type_id=3,
                    vehicle_type='Truck ',
                    make_id=None,
                    make=None
                ),
                VehicleType(
                    vehicle_type_id=7,
                    vehicle_type='Multipurpose Passenger Vehicle (MPV)',
                    make_id=None,
                    make=None
                ),
                VehicleType(
                    vehicle_type_id=9,
                    vehicle_type='Low Speed Vehicle (LSV)',
                    make_id=None,
                    make=None
                )
            ]

        """
        schema = desert.schema(VehicleType, meta=self._meta)
        results = self._client.get_vehicle_types_for_make(make)
        snake_cased = [snake_case(r) for r in results]
        return [schema.load(sc) for sc in snake_cased]

    def get_equipment_plant_codes(
        self, year: int, equipment_type: int, report_type: str = "All"
    ) -> List[PlantCode]:
        """Returns a list of plants that manufacture certain vehicle equipment.

        Plants have a unique three-character U.S. Department of Transportation
        (DOT) code. vPIC API documentation says this API only accepts 2016 and
        later.

        Args:
            year: must be 2016 or later
            equipment_type: return plants that manufacture one of these equipment
                types: 1 = Tires; 3 = Brake Hoses; 13 = Glazing; 16 = Retread
            report_type: must be one of
                New = plants whose code was assigned during the selected year
                Updated = plants whose data was modified during the selected year
                Closed = plants that are no longer active
                All = all active and closed plants, regardless of year

        Returns:
            A list of ``PlantCode``s that manufacture the equipment_type

        Raises:
            ValueError: if ``year`` is earlier than 2016

        Example:
            >>> get_equipment_plant_codes(2016, 1)
            [
                PlantCode(address='2950 INTERNATIONAL BLVD.', city='CLARKSVILLE', ...),
                PlantCode(address='1850 BARTON FERRY ROAD', city='WEST POINT', cou...),
                PlantCode(address='No. 52 Street 536, Bau Tran Hamlet, Nhuan Duc C...),
                PlantCode(address='NO. 23, HAILAR EAST ROAD', city='HUHHOT', count...),
                PlantCode(address='NO. 9 EAST BEISAN ROAD', city='SHENYANG', count...),
                PlantCode(address='HONGSHANZUI ECONOMIC DEV. ZONE', city='PINGQUAN...),
                PlantCode(address='QIANLIU VILLAGE XIADIAN TOWN', city='CHANGYI CI...),
                PlantCode(address='DOWANG TOWN', city='GUANGRAO COUNTY', country='...),
                PlantCode(address='NO.1 HUIXIN ROAD', city='GAOTANG', country='CHI...),
                PlantCode(address='NO.668 LAMEI ROAD,JIESHI TOWN,BANAN DISTRICT', ...),
            ]

        """
        plant_codes = self._client.get_equipment_plant_codes(
            year, equipment_type, report_type
        )
        schema = desert.schema(PlantCode, meta=self._meta)
        return [schema.load(snake_case(pc)) for pc in plant_codes]

    def get_models_for_make(
        self,
        make: Union[int, str],
        model_year: Optional[int] = None,
        vehicle_type: Optional[str] = None,
    ) -> List[Model]:
        """Return a list of models for a make or makes.

        Optionally filter the results by model year and vehicle type.

        Args:
            make: Pass the MakeId (int) or the complete make name (str) to return
                vehicle types for a single manufacturer. Pass a partial make name
                to return vehicle types for all makes that match the partial name.
                When you pass a make name, results will include the MakeId and
                MakeName because you may get vehicle_types for more than one make.
            model_year: pass this to return models made in this model year
            vehicle_type: one of the vPIC vehicle_types (for example, "Passenger Car",
                "Truck", or "Multipurpose Passenger Vehicle (MPV)")

        Returns:
            A list of ``Model``s

        Raises:
            ValueError: if ``year`` is earlier than 2016

        Examples:
            >>> get_models_for_make("TESLA", model_year=2020)
            [
                Model(
                    model_id=1685,
                    model='Model S',
                    make_id=441,
                    make='TESLA',
                    vehicle_type_id=None
                ),
                Model(
                    model_id=10199,
                    model='Model X',
                    make_id=441,
                    make='TESLA',
                    vehicle_type_id=None
                ),
                Model(
                    model_id=17834,
                    model='Model 3',
                    make_id=441,
                    make='TESLA',
                    vehicle_type_id=None
                ),
                Model(
                    model_id=27027,
                    model='Model Y',
                    make_id=441,
                    make='TESLA',
                    vehicle_type_id=None
                    )
            ]

        VehicleTypeId and VehicleType are only returned
        when you specify vehicle_type.

        """
        models = self._client.get_models_for_make(make, model_year, vehicle_type)
        schema = desert.schema(Model, meta=self._meta)
        return [schema.load(snake_case(m)) for m in models]

    def get_vehicle_variable_list(self) -> List[Variable]:
        """Return a list of vehicle variables tracked by vPIC

        Returns:
            A list of ``Variable``s

        Examples:
            >>> get_vehicle_variable_list()
            [
                Variable(id=1, name='Other Battery Info', data_type='string', descr...),
                Variable(id=2, name='Battery Type', data_type='lookup', description...),
                Variable(id=3, name='Bed Type', data_type='lookup', description='<p...),
                Variable(id=4, name='Cab Type', data_type='lookup', description='<p...),
                Variable(id=5, name='Body Class', data_type='lookup', description='...),
                Variable(id=9, name='Engine Number of Cylinders', data_type='int', ...),
            ]

        """
        variables = self._client.get_vehicle_variable_list()
        schema = desert.schema(Variable, meta=self._meta)
        return [schema.load(snake_case(v)) for v in variables]

    def get_vehicle_variable_values_list(self, variable_name: str) -> List[Value]:
        """Return the values for a vehicle variable

        Args:
            variable_name: the name of the vehicle variable

        Returns:
            A list of ``Values``s

        Raises:
            ValueError: if ``variable_name`` is missing

        Examples:
            >>> get_vehicle_variable_values_list("Vehicle Type")
            [
                Value(element_name="Vehicle Type", id=1, name="Motorcycle"),
                Value(element_name="Vehicle Type", id=2, name="Passenger Car"),
                Value(element_name="Vehicle Type", id=3, name="Truck "),
                Value(element_name="Vehicle Type", id=5, name="Bus"),
                Value(element_name="Vehicle Type", id=6, name="Trailer"),
                Value(element_name="Vehicle Type", id=7, name="Multipurpose P..."),
                Value(element_name="Vehicle Type", id=9, name="Low Speed Vehi..."),
                Value(element_name="Vehicle Type", id=10, name="Incomplete Ve..."),
                Value(element_name="Vehicle Type", id=13, name="Off Road Vehicle")
            ]

        """
        values = self._client.get_vehicle_variable_values_list(variable_name)
        schema = desert.schema(Value, meta=self._meta)
        return [schema.load(snake_case(v)) for v in values]



from requests import Session


class VpicAPISession(Session):
    def __init__(self, *args, **kwargs):
        super(VpicAPISession, self).__init__(*args, **kwargs)

        self.headers.update(
            {
                "Accept-Charset": "utf-8",
            }
        )


__all__ = ["VpicAPISession"]



from vpic.client_base import ClientBase


def test_init():
    c = ClientBase(host="http://somehost/")

    assert c.host == "http://somehost/"


def test_url():
    c = ClientBase(host="http://somehost")

    assert c.url == "http://somehost"



from dataclasses import dataclass
from typing import List, Optional


@dataclass(eq=True, frozen=True)
class Document:
    cover_letter_url: str
    letter_date: str
    manufacturer_id: int
    manufacturer: str
    name: str
    url: str
    type: Optional[str] = None
    model_year_from: Optional[int] = None
    model_year_to: Optional[int] = None


@dataclass(eq=True, frozen=True)
class Make:
    make_id: int
    make: str
    manufacturer_id: Optional[int] = None
    manufacturer: Optional[str] = None
    vehicle_type_id: Optional[int] = None
    vehicle_type: Optional[str] = None


@dataclass(eq=True, frozen=True)
class Model:
    model_id: int
    model: str
    make_id: Optional[int] = None
    make: Optional[str] = None
    vehicle_type_id: Optional[int] = None
    vehicle_type: Optional[str] = None


@dataclass(eq=True, frozen=True)
class Manufacturer:
    manufacturer_id: int
    manufacturer: str
    manufacturer_common_name: str


@dataclass(eq=True, frozen=True)
class ManufacturerType:
    name: str


@dataclass(eq=True, frozen=False)
class VehicleType:
    name: Optional[str]
    vehicle_type: Optional[str] = None
    vehicle_type_id: Optional[int] = None
    make: Optional[str] = None
    make_id: Optional[int] = None
    gvwr_from: Optional[str] = None
    gvwr_to: Optional[str] = None
    is_primary: Optional[bool] = None

    def __post_init__(self):
        if self.name is not None:
            self.vehicle_type = self.name
        del self.name


@dataclass(eq=True, frozen=True)
class ManufacturerDetail:
    manufacturer_id: int
    manufacturer: str
    manufacturer_common_name: Optional[str]
    manufacturer_types: List[ManufacturerType]
    vehicle_types: List[VehicleType]
    address: Optional[str] = None
    address2: Optional[str] = None
    city: Optional[str] = None
    contact_email: Optional[str] = None
    contact_fax: Optional[str] = None
    contact_phone: Optional[str] = None
    country: Optional[str] = None
    dbas: Optional[str] = None
    equipment_items: Optional[List[str]] = None
    last_updated: Optional[str] = None
    other_manufacturer_details: Optional[str] = None
    postal_code: Optional[str] = None
    primary_product: Optional[str] = None
    principal_first_name: Optional[str] = None
    principal_last_name: Optional[str] = None
    principal_position: Optional[str] = None
    state_province: Optional[str] = None
    submitted_name: Optional[str] = None
    submitted_on: Optional[str] = None
    submitted_position: Optional[str] = None


@dataclass(eq=True, frozen=True)
class PlantCode:
    dot_code: str
    name: str
    address: Optional[str] = None
    city: Optional[str] = None
    country: Optional[str] = None
    old_dot_code: Optional[str] = None
    postal_code: Optional[str] = None
    state_province: Optional[str] = None
    status: Optional[str] = None


@dataclass(eq=True, frozen=True)
class Value:
    element_name: str
    id: int
    name: str


@dataclass(eq=True, frozen=True)
class Variable:
    id: int
    name: str
    group_name: Optional[str]
    data_type: str
    description: str


@dataclass(frozen=True)
class Vehicle:
    abs: str
    active_safety_sys_note: str
    adaptive_cruise_control: str
    adaptive_driving_beam: str
    adaptive_headlights: str
    additional_error_text: str
    air_bag_loc_curtain: str
    air_bag_loc_front: str
    air_bag_loc_knee: str
    air_bag_loc_seat_cushion: str
    air_bag_loc_side: str
    auto_reverse_system: str
    automatic_pedestrian_alerting_sound: str
    axle_configuration: str
    axles: str
    base_price: str
    battery_a: str
    battery_a_to: str
    battery_cells: str
    battery_info: str
    battery_kwh: str
    battery_kwh_to: str
    battery_modules: str
    battery_packs: str
    battery_type: str
    battery_v: str
    battery_v_to: str
    bed_length_in: str
    bed_type: str
    blind_spot_intervention: str
    blind_spot_mon: str
    body_cab_type: str
    body_class: str
    brake_system_desc: str
    brake_system_type: str
    bus_floor_config_type: str
    bus_length: str
    bus_type: str
    can_aacn: str
    cib: str
    cash_for_clunkers: str
    charger_level: str
    charger_power_kw: str
    cooling_type: str
    curb_weight_lb: str
    custom_motorcycle_type: str
    daytime_running_light: str
    destination_market: str
    displacement_cc: str
    displacement_ci: str
    displacement_l: str
    doors: str
    drive_type: str
    driver_assist: str
    dynamic_brake_support: str
    edr: str
    esc: str
    ev_drive_unit: str
    electrification_level: str
    engine_configuration: str
    engine_cycles: str
    engine_cylinders: str
    engine_hp: str
    engine_hp_to: str
    engine_kw: str
    engine_manufacturer: str
    engine_model: str
    entertainment_system: str
    error_code: str
    error_text: str
    forward_collision_warning: str
    fuel_injection_type: str
    fuel_type_primary: str
    fuel_type_secondary: str
    gcwr_from: str
    gcwr_to: str
    gvwr_from: str
    gvwr_to: str
    keyless_ignition: str
    lane_centering_assistance: str
    lane_departure_warning: str
    lane_keep_system: str
    lower_beam_headlamp_light_source: str
    make: str
    make_id: str
    manufacturer: str
    manufacturer_id: str
    model: str
    model_id: str
    model_year: str
    motorcycle_chassis_type: str
    motorcycle_suspension_type: str
    ncsa_body_type: str
    ncsa_make: str
    ncsa_map_exc_approved_by: str
    ncsa_map_exc_approved_on: str
    ncsa_mapping_exception: str
    ncsa_model: str
    ncsa_note: str
    note: str
    other_bus_info: str
    other_engine_info: str
    other_motorcycle_info: str
    other_restraint_system_info: str
    other_trailer_info: str
    park_assist: str
    pedestrian_automatic_emergency_braking: str
    plant_city: str
    plant_company_name: str
    plant_country: str
    plant_state: str
    possible_values: str
    pretensioner: str
    rear_automatic_emergency_braking: str
    rear_cross_traffic_alert: str
    rear_visibility_system: str
    sae_automation_level: str
    sae_automation_level_to: str
    seat_belts_all: str
    seat_rows: str
    seats: str
    semiautomatic_headlamp_beam_switching: str
    series: str
    series2: str
    steering_location: str
    suggested_vin: str
    tpms: str
    top_speed_mph: str
    track_width: str
    traction_control: str
    trailer_body_type: str
    trailer_length: str
    trailer_type: str
    transmission_speeds: str
    transmission_style: str
    trim: str
    trim2: str
    turbo: str
    vin: str
    valve_train_design: str
    vehicle_type: str
    wheel_base_long: str
    wheel_base_short: str
    wheel_base_type: str
    wheel_size_front: str
    wheel_size_rear: str
    wheels: str
    windows: str


@dataclass(eq=True, frozen=False)
class WMI:
    manufacturer: Optional[str]
    name: Optional[str]
    id: Optional[int]
    vehicle_type: str
    wmi: Optional[str] = None
    common_name: Optional[str] = None
    url: Optional[str] = None
    country: Optional[str] = None
    make: Optional[str] = None
    manufacturer_id: Optional[int] = None
    parent_company_name: Optional[str] = None
    created_on: Optional[str] = None
    updated_on: Optional[str] = None
    date_available_to_public: Optional[str] = None

    def __post_init__(self):
        if self.id is not None:
            self.manufacturer_id = self.id
        del self.id
        if self.name is not None:
            self.manufacturer = self.name
        del self.name



from typing import List
from vpic.models import Variable, Value
from vpic.typed_client import TypedClient


class TestVariables:
    def test_get_vehicle_variable_list(self, typed_client: TypedClient, responses):
        variables: List[Variable] = typed_client.get_vehicle_variable_list()
        assert len(variables) == 138
        assert hasattr(variables[0], "data_type")
        assert hasattr(variables[0], "description")
        assert hasattr(variables[0], "group_name")
        assert hasattr(variables[0], "id")
        assert hasattr(variables[0], "name")
        assert variables[0].id == 1

    def test_get_vehicle_variable_values_list(
        self, typed_client: TypedClient, responses
    ):
        values: List[Value] = typed_client.get_vehicle_variable_values_list(
            "Vehicle Type"
        )
        assert len(values) == 9
        assert values[0].id == 1
        assert values[0].name == "Motorcycle"



import logging
from typing import Any, Dict, Optional
from urllib.parse import urljoin

from requests.adapters import HTTPAdapter

from .exceptions import handle_error_response
from .session import VpicAPISession
from .transforms import standardize

log: logging.Logger = logging.getLogger(__name__)


class ClientBase(object):
    """A base class for vPIC clients

    Args:
        host: Hostname, including http(s)://, of the vPIC instance to query
        standardize_variables: vPIC uses different names for the same
            variable. Set this to True to standardize variables before
            returning the response.

    """

    def __init__(
        self,
        host: Optional[str] = "https://vpic.nhtsa.dot.gov/api/vehicles",
        standardize_variables: bool = True,
    ):
        if host:
            self.host: str = host
        self.standardize_variables: bool = standardize_variables
        self.session = VpicAPISession()
        self.session.mount(self.url, HTTPAdapter(pool_connections=2, max_retries=5))

    @property
    def url(self) -> str:
        return self.host

    def _request(self, endpoint: str, params: Dict[str, Any] = None):
        if not params:
            params = {"format": "json"}
        else:
            params["format"] = "json"

        api = urljoin(self.url, endpoint)
        resp = self.session.get(api, params=params)

        if resp.status_code >= 400:
            handle_error_response(resp)

        results = resp.json()["Results"]
        if self.standardize_variables:
            results = standardize(results)

        return results

    def _request_post(self, endpoint: str, data, params=None):
        if not params:
            params = {"format": "json"}
        elif "format" not in params:
            params["format"] = "json"

        api = urljoin(self.url, endpoint)
        resp = self.session.post(url=api, data=data, params=params)

        if resp.status_code >= 400:
            handle_error_response(resp)

        results = resp.json()["Results"]
        if self.standardize_variables:
            results = standardize(results)

        return results



import logging
from typing import Any, Dict, List, Optional, Union

from .client_base import ClientBase

log = logging.getLogger(__name__)


class Client(ClientBase):
    """A client library for the U.S. NHTSA vPIC API

    ``Client`` returns JSON responses from the vPIC API. vPIC responses
    don't always use the same name for a variable, so by default this
    library standardizes variable names. You can disable this by creating
    a client like this:

        ``c = Client(standardize_names=False)``

    If you prefer to receive model objects instead of JSON responses,
    use ``vpic.Client`` instead.

    A client library for the United States National Highway Traffic Safety
    Administration (NHTSA) Vehicle Product Information Catalog (vPIC) Vehicle
    Listing API.

    Use this to gather information on vehicles and their specifications,
    and to decode VINs to extract information for specific vehicles. vPIC
    has information about these types of vehicles sold or imported in
    the USA:

    * Bus
    * Incomplete Vehicle
    * Low Speed Vehicle (LSV)
    * Motorcycle
    * Multipurpose Passenger Vehicle (MPV)
    * Passenger Car
    * Trailer
    * Truck

    vPIC has all of the information about how manufacturers assign a VIN that
    encodes the vehicles characteristics. Vehicle manufacturers provide this
    information to NHTSA under U.S. law 49 CFR Part 565.

    The API available 24/7, is free to use, and does not require registration.
    NHTSA uses automatic traffic rate controls to maintain the performance of
    the API and their websites that use the API.

    See https://vpic.nhtsa.dot.gov/api for more on the API.

    Attributes:
        host: Hostname, including http(s)://, of the vPIC instance to query.
        standardize_variables: vPIC uses different names for the same
            variable, so this client standarizes those names by default.
            Set this to False to receive the raw vPIC response.

    """

    def __init__(
        self,
        host: Optional[str] = "https://vpic.nhtsa.dot.gov/api/vehicles/",
        standardize_variables: bool = True,
    ):
        super(Client, self).__init__(host, standardize_variables)

    def decode_vin(
        self, vin: str, model_year: int = None, extend=False, flatten=True
    ) -> Dict[str, Any]:
        """Decode a 17-digit Vehicle Identification Number (VIN) or partial VIN.

        Decode the make, model, series, trim, and other vehicle information
        from VIN. Model year is required for pre-1980 vehicles, though vPIC
        recommends that you always pass it.

        If you don't have a complete 17-digit VIN, you can pass a partial
        VIN, using asterisk (*) for missing characters. The VIN check digit
        (the 9th character) isn't required for partial VINs. The shorter the
        partial VIN, the less vehicle information you'll receive in the
        response.

        See get_vehicle_variable_list for the variables returned here.

        Args:
            vin: A 17-digit VIN or partial VIN. Use asterisk for missing
                characters.
            model_year: The vehicle's model year. Recommended, but not required.
            extend: If True, response will include variables for other NHTSA
                programs like NCSA. Defaults to False.
            flatten: True to receive vehicle variables in key-value pairs (this is
                the default and usually best choice). False to receive them as a
                list of variable objects that include the variable ID.

        Raises:
            ValueError: if ``vin`` is missing or isn't 6 to 17 characters long.
            ValueError: if ``model_year`` is earlier than 1981.

        """
        if vin is None:
            raise ValueError("vin is required")
        if not len(vin) in range(6, 17 + 1):
            raise ValueError(
                "vin must be at least 6 characters and at most 17 characters"
            )
        if model_year and model_year < 1981:
            raise ValueError("model year must be 1981 or later")

        endpoint = "DecodeVin"
        if flatten:
            endpoint = "DecodeVinValues"
        if extend:
            endpoint = f"{endpoint}Extended"

        if model_year is not None:
            params = {"modelyear": model_year}
        else:
            params = {}

        results = self._request(f"{endpoint}/{vin}", params)
        return results[0] if flatten else results

    def decode_vin_batch(self, vins: List[str]) -> List[Dict[str, Any]]:
        """Decode a batch of 17-digit VINs or partial VINs.

        Model year is required for pre-1980 vehicles, though vPIC recommends
        that you always pass it.

        If you don't have a complete 17-digit VIN, you can pass a partial
        VIN, using asterisk (*) for missing characters. The VIN check digit
        (the 9th character) isn't required for partial VINs. The shorter the
        partial VIN, the less vehicle information you'll receive in the
        response.

        Vehicle variables will be returned in key-value pairs, the same
        format returned by decode_vin(.., flatten=True).

        See get_vehicle_variable_list for the variables returned here.

        Args:
            vins: A list of 17-digit VIN or partial VINs and optional model year.
                Use asterisk for missing characters. For example: ["VIN, model_year",
                "VIN, model_year", ...]

        Raises:
            ValueError: if ``vin`` is missing or isn't 6 to 17 characters long.
            ValueError: if ``model_year`` is earlier than 1981.

        """
        if vins is None:
            raise ValueError("vins is required")

        if not len(vins) in range(1, 50 + 1):
            raise ValueError("pass at least one VIN, and at most 50 VINs")

        return self._request_post("DecodeVINValuesBatch", data={"DATA": ";".join(vins)})

    def decode_wmi(self, wmi: str) -> Dict[str, Any]:
        """Decode a WMI to get manufacturer information

        Provides information on the World Manufacturer Identifier for a
        specific WMI code.

        Args:
            wmi: A 3-character or 6-character World Manufacturer Index code.
                 Large volume manufacturers usually have a  3 character WMI
                 representing positions 1 to 3 ("JTD") of a VIN. Smaller
                 manufacturers have a 6 character WMI representing positions
                 1 to 3 and 12 to 14 of a VIN.

        Raises:
            ValueError: if ``wmi`` is missing or isn't 3 or 6 characters long.

        Example:
            >>> decode_wmi('1FT')
            {
                "CommonName": "Ford",
                "CreatedOn": "2015-03-23",
                "DateAvailableToPublic": "2015-01-01",
                "MakeName": "FORD",
                "ManufacturerName": "FORD MOTOR COMPANY, USA",
                "ParentCompanyName": "",
                "URL": "http://www.ford.com/",
                "UpdatedOn": null,
                "VehicleType": "Truck ",
            }

        """
        if not len(wmi) in [3, 6]:
            raise ValueError("WMI must be 3 or 6 characters")

        result = self._request(f"DecodeWMI/{wmi}")[0]
        # result["WMI"] = wmi
        return result

    def get_wmis_for_manufacturer(
        self,
        manufacturer: Optional[Union[str, int]] = None,
        vehicle_type: Optional[Union[str, int]] = None,
    ) -> List[Dict[str, Any]]:
        """Returns the WMIs for one or all manufacturers

        You must pass one or both of provide manufacturer or vehicle_type.

        Args:
            manufacturer: Pass the Manufacturer Id (int) or the complete
                manufacturer name (str) to return WMIs for a single manufacturer.
                Pass a partial name to return WMIs for all manufacturers with
                names that include the partial name.
            vehicle_type: Pass the vehicle_type Id (int) or complete vehicle_type
                name to return WMIs for that vehicle_type. Pass a partial name to
                return WMIs for vehicle_types matching that name.

        Raises:
            ValueError: if ``manufacturer`` and ``vehicle_type`` are missing

        Examples:
            >>> get_wmis_for_manufacturer('Honda')
            [
                {
                    "Country": null,
                    "CreatedOn": "2015-03-26",
                    "DateAvailableToPublic": "2015-01-01",
                    "Id": 987,
                    "Name": "HONDA MOTOR CO., LTD",
                    "UpdatedOn": "2015-06-04",
                    "VehicleType": "Passenger Car",
                    "WMI": "JHM"
                },
                ...
            ]

        """
        if manufacturer is None and vehicle_type is None:
            raise ValueError("manufacturer or vehicle_type is required")

        if manufacturer is None:
            endpoint = "GetWMIsForManufacturer"
        else:
            endpoint = f"GetWMIsForManufacturer/{manufacturer}"

        params = {}
        if vehicle_type:
            params["vehicleType"] = vehicle_type

        wmis = self._request(endpoint, params)

        # for wmi in wmis:
        #     wmi["ManufacturerId"] = wmi["Id"]
        #     del wmi["Id"]
        #     wmi["Manufacturer"] = wmi["Name"]
        #     del wmi["Name"]

        return wmis

    def get_all_makes(self) -> List[Dict[str, Any]]:
        """Returns all of the makes registered with vPIC.

        Examples:
            >>> get_all_makes()
            [
                {
                    "MakeId": 440,
                    "MakeName": "ASTON MARTIN"
                },
                {
                    "MakeId": 441,
                    "MakeName": "TESLA"
                },
                {
                    "MakeId": 442,
                    "MakeName": "JAGUAR"
                },
                ...
            ]

        """
        return self._request("GetAllMakes")

    def get_parts(
        self, cfr_part: str, from_date: str, to_date: str, page: int = 1
    ) -> List[Dict[str, Any]]:
        """Returns a list of vehicle documentation submitted by manufacturers.

        Manufacturers provide vehicle information to NHTSA to comply with these
        regulations:

        * 49 CFR Part 565 (Vehicle Identification Number Guidance)
        * 49 CFR Part 566 (Manufacturer Identification – Reporting Requirements)

        This provides a list of documents submitted in a date range. Up to 1,000
        results will be returned at a time.

        Args:
            cfr_part: '565' to return 49 CFR Part 565 submissions;
                '566' to return 49 CFR Part 566 submissions
            from_date: the beginning of the date range to search
            end_date: the end of the date range to search
            page: results are paginated; this is page number to return

        Raises:
            ValueError: if ``cfr_part`` is missing

        Examples:
            >>> get_parts('565', '2015-01-01', '2015-05-05', 1)
            [
                {
                    "CoverLetterURL": "",
                    "LetterDate": "5/5/2015",
                    "ManufacturerId": 8012,
                    "ManufacturerName": "PORSCHE CARS NORTH AMERICA, INC.",
                    "ModelYearFrom": null,
                    "ModelYearTo": null,
                    "Name": "ORG10658",
                    "Type": null,
                    "URL": "http://vpic.nhtsa.dot.gov/mid/home/displayfile/[guid here]"
                },
                ...
            ]

        """
        if cfr_part is None:
            raise ValueError("cfr_part is required")

        params = {
            "type": cfr_part,
            "fromDate": from_date,
            "toDate": to_date,
            "page": page,
        }
        return self._request("GetParts", params)

    def get_all_manufacturers(
        self, manufacturer_type: str = None, page: int = 1
    ) -> List[Dict[str, Any]]:
        """Return a list of vPIC manufacturers of the given manufacturer_type.

        This provides a list of all the Manufacturers available in vPIC Dataset.

        See ``get_vehicle_variable_values_list("Manufacturer Type")`` for the list
        of manufacturer types.

        Args:
            manufacturer_type: The manufacturer type, which is Incomplete Vehicles,
                Completed Vehicle Manufacturer, Incomplete Vehicle Manufacturer,
                Intermediate Manufacturer, Final-Stage Manufacturer, Alterer,
                Replica Vehicle Manufacturer. You can pass the full type name, or a
                substring of the type.
            page: results are paginated; this is the page number to return

        Examples:
            >>> get_all_manufacturers("Completed Vehicle", 1)
            [
                {
                "Country": "UNITED STATES (USA)",
                "Mfr_CommonName": "Tesla",
                "Mfr_ID": 955,
                "Mfr_Name": "TESLA, INC.",
                "VehicleTypes": [
                    {
                        "IsPrimary": true,
                        "Name": "Passenger Car"
                    },
                    {
                        "IsPrimary": false,
                        "Name": "Multipurpose Passenger Vehicle (MPV)"
                    }
                },
                ...
            ]

        """
        params = {"ManufacturerType": manufacturer_type, "page": page}
        return self._request("GetAllManufacturers", params)

    def get_manufacturer_details(
        self, manufacturer: Union[str, int]
    ) -> List[Dict[str, Any]]:
        """Returns details for one or more manufacturers.

        Args:
            manufacturer: Pass the Manufacturer Id (int) or the complete
                manufacturer name (str) to return detail for a single
                manufacturer. Pass a partial name to return manufacturers
                with names that include the partial name.

        Examples:
            >>> get_manufacturer_details(988)
            [
                {
                    "Address": "1919 Torrance Blvd.",
                    "Address2": null,
                    "City": "Torrance",
                    "ContactEmail": "jeff_chang@ahm.honda.com",
                    "ContactFax": null,
                    "ContactPhone": "(310)783-3401",
                    "Country": "UNITED STATES (USA)",
                    "DBAs": "...",
                    "EquipmentItems": [],
                    "LastUpdated": "/Date(1618422117803-0400)/",
                    "ManufacturerTypes": [
                        {
                            "Name": "Completed Vehicle Manufacturer"
                        }
                    ],
                    "Mfr_CommonName": "Honda",
                    "Mfr_ID": 988,
                    "Mfr_Name": "HONDA DEVELOPMENT & MANUFACTURING OF AMERICA, LLC",
                    "OtherManufacturerDetails": null,
                    "PostalCode": "90501",
                    "PrimaryProduct": null,
                    "PrincipalFirstName": "Shinji Aoyama",
                    "PrincipalLastName": null,
                    "PrincipalPosition": "President & CEO",
                    "StateProvince": "CALIFORNIA",
                    "SubmittedName": "Wilson Tran",
                    "SubmittedOn": "/Date(1618286400000-0400)/",
                    "SubmittedPosition": "Sr. Specialist, II",
                    "VehicleTypes": [
                        {
                            "GVWRFrom": "Class 1A: 3,000 lb or less (1,360 kg or less)",
                            "GVWRTo": "Class 1D: 5,001 - 6,000 lb (2,268 - 2,722 kg)",
                            "IsPrimary": true,
                            "Name": "Passenger Car"
                        },
                        {
                            "GVWRFrom": "Class 2E: 6,001 - 7,000 lb (2,722 - 3,175 kg)",
                            "GVWRTo": "Class 2E: 6,001 - 7,000 lb (2,722 - 3,175 kg)",
                            "IsPrimary": false,
                            "Name": "Truck "
                        },
                        {
                            "GVWRFrom": "Class 1B: 3,001 - 4,000 lb (1,360 - 1,814 kg)",
                            "GVWRTo": "Class 2E: 6,001 - 7,000 lb (2,722 - 3,175 kg)",
                            "IsPrimary": false,
                            "Name": "Multipurpose Passenger Vehicle (MPV)"
                        }
                    ]
                }
                ...
            ]

        """
        if manufacturer is None:
            raise ValueError("manufacturer is required")

        return self._request(f"GetManufacturerDetails/{manufacturer}")

    def get_makes_for_manufacturer(
        self, manufacturer: Union[str, int], model_year: int = None
    ) -> List[Dict[str, Any]]:
        """Returns makes produced by a manufacturer or manufacturers.

        Args:
            manufacturer: Pass the Manufacturer Id (int) or the complete
                manufacturer name (str) to return detail for a single manufacturer.
                Pass a partial name to return manufacturers with names that include
                the partial name.
            model_year: Pass a model year to return only those makes made by
                the manufacturer for that model year.

        Raises:
            ValueError: if ``manufacturer`` is missing

        Examples:
            >>> get_makes_for_manufacturer(988)
            [
                {
                    "MakeId": 474,
                    "MakeName": "HONDA",
                    "Mfr_Name": "HONDA DEVELOPMENT & MANUFACTURING OF AMERICA, LLC"
                },
                {
                    "MakeId": 475,
                    "MakeName": "ACURA",
                    "Mfr_Name": "HONDA DEVELOPMENT & MANUFACTURING OF AMERICA, LLC"
                }
                ...
            ]

        """
        if manufacturer is None:
            raise ValueError("manufacturer is required")

        if model_year:
            results = self._request(
                f"GetMakesForManufacturerAndYear/{manufacturer}", {"year": model_year}
            )
        else:
            results = self._request(f"GetMakeForManufacturer/{manufacturer}")

        return results

    def get_makes_for_vehicle_type(self, vehicle_type: str) -> List[Dict[str, Any]]:
        """Returns makes that produce a vehicle_type

        Args:
            vehicle_type: A vPIC vehicle_type. For example, "Passenger Car",
                "Truck", or "Multipurpose Passenger Vehicle (MPV)". If you pass
                a partial vehicle_type, for example "Passenger", results will
                include makes for all matching vehicle types. Matching is not
                case sensitive.

        Raises:
            ValueError: if ``vehicle_type`` is missing

        Examples:
            >>> get_makes_for_vehicle_type('Car')
            [
                {
                    "MakeId": 440,
                    "MakeName": "ASTON MARTIN",
                    "VehicleTypeId": 2,
                    "VehicleTypeName": "Passenger Car"
                },
                {
                    "MakeId": 441,
                    "MakeName": "TESLA",
                    "VehicleTypeId": 2,
                    "VehicleTypeName": "Passenger Car"
                },
                ...
            ]

        """
        if vehicle_type is None:
            raise ValueError("vehicle_type is required")

        return self._request(f"GetMakesForVehicleType/{vehicle_type}")

    def get_vehicle_types_for_make(self, make: Union[str, int]) -> List[Dict[str, Any]]:
        """Returns vehicle types produced by a make or make

        Args:
            make: Pass the MakeId (int) or the complete make name (str) to return
                vehicle types for a single manufacturer. Pass a partial make name
                to return vehicle types for all makes that match the partial name.
                When you pass a make name, results will include the MakeId and
                MakeName because you may get vehicle_types for more than one make.

        Raises:
            ValueError: if ``make`` is missing

        Examples:
            >>> get_vehicle_types_for_make(474)
            [
                {
                    "VehicleTypeId": 1,
                    "VehicleTypeName": "Motorcycle"
                },
                {
                    "VehicleTypeId": 2,
                    "VehicleTypeName": "Passenger Car"
                },
                {
                    "VehicleTypeId": 3,
                    "VehicleTypeName": "Truck "
                },
                {
                    "VehicleTypeId": 7,
                    "VehicleTypeName": "Multipurpose Passenger Vehicle (MPV)"
                },
                {
                    "VehicleTypeId": 9,
                    "VehicleTypeName": "Low Speed Vehicle (LSV)"
                }
            ]

            >>> get_vehicle_types_for_make('kia')
            [
                {
                    "MakeId": 499,
                    "MakeName": "KIA",
                    "VehicleTypeId": 2,
                    "VehicleTypeName": "Passenger Car"
                },
                {
                    "MakeId": 499,
                    "MakeName": "KIA",
                    "VehicleTypeId": 7,
                    "VehicleTypeName": "Multipurpose Passenger Vehicle (MPV)"
                },
                {
                    "MakeId": 5848,
                    "MakeName": "MGS GRAND SPORT (MARDIKIAN)",
                    "VehicleTypeId": 2,
                    "VehicleTypeName": "Passenger Car"
                }
            ]

        """
        if make is None:
            raise ValueError("make is required")

        if isinstance(make, int):
            return self._request(f"GetVehicleTypesForMakeId/{make}")
        else:
            return self._request(f"GetVehicleTypesForMake/{make}")

    def get_equipment_plant_codes(
        self, year: int, equipment_type: int, report_type: str = "All"
    ) -> List[Dict[str, Any]]:
        """Returns a list of plants that manufacture certain vehicle equipment.

        Plants have a unique three-character U.S. Department of Transportation
        (DOT) code. vPIC API documentation says this API only accepts 2016 and
        later.

        Args:
            year: must be 2016 or later
            equipment_type: return plants that manufacture one of these equipment
                types: 1 = Tires; 3 = Brake Hoses; 13 = Glazing; 16 = Retread
            report_type: must be one of
                New = plants whose code was assigned during the selected year
                Updated = plants whose data was modified during the selected year
                Closed = plants that are no longer active
                All = all active and closed plants, regardless of year

        Raises:
            ValueError: if ``year`` is earlier than 2016

        Example:
            >>> get_equipment_plant_codes(2016, 1)
            [
                {
                    "Address": "2950 INTERNATIONAL BLVD.",
                    "City": "CLARKSVILLE",
                    "Country": "USA",
                    "DOTCode": "00T",
                    "Name": "HANKOOK TIRE MANUFACTURING TENNESSEE, LP",
                    "OldDotCode": "",
                    "PostalCode": "37040",
                    "StateProvince": "TENNESSEE",
                    "Status": "Active"
                },
                ...
            ]

        """
        if year < 2016:
            raise ValueError("Year must be 2016 or later")

        params = {
            "year": year,
            "equipmentType": equipment_type,
            "reportType": report_type,
        }
        return self._request("GetEquipmentPlantCodes", params)

    def get_models_for_make(
        self, make: Union[int, str], model_year: int = None, vehicle_type: str = None
    ) -> List[Dict[str, Any]]:
        """Return a list of models for a make or makes.

        Optionally filter the results by model year and vehicle type.

        Args:
            make: Pass the MakeId (int) or the complete make name (str) to return
                vehicle types for a single manufacturer. Pass a partial make name
                to return vehicle types for all makes that match the partial name.
                When you pass a make name, results will include the MakeId and
                MakeName because you may get vehicle_types for more than one make.
            model_year: pass this to return models made in this model year
            vehicle_type: one of the vPIC vehicle_types (for example, "Passenger Car",
                "Truck", or "Multipurpose Passenger Vehicle (MPV)")

        Raises:
            ValueError: if ``year`` is earlier than 2016

        Examples:
            >>> get_models_for_make("TESLA", model_year=2020)
            [
                {
                    "MakeId": 441,
                    "MakeName": "TESLA",
                    "ModelId": 1685,
                    "ModelName": "Model S"
                },
                {
                    "MakeId": 441,
                    "MakeName": "TESLA",
                    "ModelId": 10199,
                    "ModelName": "Model X"
                },
                {
                    "MakeId": 441,
                    "MakeName": "TESLA",
                    "ModelId": 17834,
                    "ModelName": "Model 3"
                },
                {
                    "MakeId": 441,
                    "MakeName": "TESLA",
                    "ModelId": 27027,
                    "ModelName": "Model Y"
                }
            ]

        VehicleTypeId and VehicleTypeName are only returned
        when you specify vehicle_type.

        """
        if make is None:
            raise ValueError("make is required")

        if model_year or vehicle_type:
            my = f"/modelyear/{model_year}" if model_year else ""
            vt = f"/vehicletype/{vehicle_type}" if vehicle_type else ""
            if isinstance(make, int):
                endpoint = f"GetModelsForMakeIdYear/makeId/{make}{my}{vt}"
            else:
                endpoint = f"GetModelsForMakeYear/make/{make}{my}{vt}"
        else:
            if isinstance(make, int):
                endpoint = f"GetModelsForMakeId/{make}"
            else:
                endpoint = f"GetModelsForMake/{make}"

        return self._request(endpoint)

    def get_vehicle_variable_list(self) -> List[Dict[str, Any]]:
        """Return a list of vehicle variables tracked by vPIC

        Examples:
            >>> get_vehicle_variable_list()
            [
                {
                    "DataType": "string",
                    "Description": "<p>Any other battery information that does...",
                    "Id": 1,
                    "Name": "Other Battery Info"
                },
                {
                    "DataType": "lookup",
                    "Description": "<p>Battery type field stores the battery ...",
                    "Id": 2,
                    "Name": "Battery Type"
                },
                {
                    "DataType": "lookup",
                    "Description": "<p>Bed type is the type of bed (the open b...",
                    "Id": 3,
                    "Name": "Bed Type"
                },
                {
                    "DataType": "lookup",
                    "Description": "<p>Cab type applies to both pickup truck ...",
                    "Id": 4,
                    "Name": "Cab Type"
                },
                {
                    "DataType": "lookup",
                    "Description": "<p>Body Class presents the Body Type, bas...",
                    "Id": 5,
                    "Name": "Body Class"
                },
                ...
            ]

        """
        return self._request("GetVehicleVariableList")

    def get_vehicle_variable_values_list(
        self, variable_name: str
    ) -> List[Dict[str, Any]]:
        """Return the values for a vehicle variable

        Args:
            variable_name: the name of the vehicle variable

        Raises:
            ValueError: if ``variable_name`` is missing

        Examples:
            >>> get_vehicle_variable_values_list("Vehicle Type")
            [
                {
                    "ElementName": "Vehicle Type",
                    "Id": 1,
                    "Name": "Motorcycle"
                },
                {
                    "ElementName": "Vehicle Type",
                    "Id": 2,
                    "Name": "Passenger Car"
                },
                {
                    "ElementName": "Vehicle Type",
                    "Id": 3,
                    "Name": "Truck "
                },
                {
                    "ElementName": "Vehicle Type",
                    "Id": 5,
                    "Name": "Bus"
                },
                {
                    "ElementName": "Vehicle Type",
                    "Id": 6,
                    "Name": "Trailer"
                },
                {
                    "ElementName": "Vehicle Type",
                    "Id": 7,
                    "Name": "Multipurpose Passenger Vehicle (MPV)"
                },
                {
                    "ElementName": "Vehicle Type",
                    "Id": 9,
                    "Name": "Low Speed Vehicle (LSV)"
                },
                {
                    "ElementName": "Vehicle Type",
                    "Id": 10,
                    "Name": "Incomplete Vehicle"
                },
                {
                    "ElementName": "Vehicle Type",
                    "Id": 13,
                    "Name": "Off Road Vehicle"
                }
            ]

        """
        if variable_name is None:
            raise ValueError("variable_name is required")

        return self._request(f"GetVehicleVariableValuesList/{variable_name}")

    def get_canadian_vehicle_specifications(
        self, year: int, make: str, model: str = None, units: str = "Metric"
    ) -> List[Dict[str, Any]]:
        """Get original vehicle dimensions from the Canadian Vehicle Specification.

        The Canadian Vehicle Specifications (CVS) consists of a database of
        original vehicle dimensions, used primarily in collision investigation
        and reconstruction, combined with a search engine. The database is
        compiled annually by the Collision Investigation and Research Division
        of Transport Canada.

        See [Canadian Vehicle Specifications](http://www.carsp.ca/research/resources
        /safety-sources/canadian-vehicle-specifications/).

        Args:
            year: 1971 or later
            make: a make name like "Honda", "Toyota", ...
            model: a model name like "Pilot", "Focus", ...
            units: "Metric" or "US"

        """
        params = {"Year": year, "Make": make, "Model": model, "units": units}
        return self._request("GetCanadianVehicleSpecifications", params=params)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['vpic']

package_data = \
{'': ['*']}

install_requires = \
['desert>=2020.11.18,<2021.0.0', 'requests>=2.25.1,<3.0.0']

setup_kwargs = {
    'name': 'vpic-api',
    'version': '0.7.3',
    'description': 'A client library for the United States National Highway Traffic Safety Administration (NHTSA) Vehicle Product Information Catalog (vPIC) API',
    'long_description': '# vPIC API Client Library\n\n![PyPI](https://img.shields.io/pypi/v/vpic-api)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/vpic-api)\n![PyPI - Implementation](https://img.shields.io/pypi/implementation/vpic-api)\n![PyPI - Wheel](https://img.shields.io/pypi/wheel/vpic-api)\n[![MIT License](https://img.shields.io/apm/l/atomic-design-ui.svg?)](https://github.com/tterb/atomic-design-ui/blob/master/LICENSEs)\n[![Tests](https://github.com/davidpeckham/vpic-api/actions/workflows/main.yml/badge.svg)](https://github.com/davidpeckham/vpic-api/actions/workflows/main.yml)\n\nA Python client library for decoding VINs and querying the United States \nNational Highway Traffic Safety Administration (NHTSA) [Vehicle Product \nInformation Catalog Vehicle Listing (vPIC) API](https://vpic.nhtsa.dot.gov/api/).\n\n## NHTSA vPIC API\n\nThe Vehicle Production Information Catalog (vPIC) API is hosted by the U.S. National Highway Transportation Safety Administration (NHTSA). vPIC data is provided by manufacturers who make vehicles for sale in the United States. The vPIC API is available 24/7, is free to use, and does not require registration. See https://vpic.nhtsa.dot.gov/api/home/index/faq for more on the vPIC API.\n\n## Features\n\n- Decode 17-digit Vehicle Identification Numbers (VIN)\n- Decode a partial VIN\n- Discover manufacturers, makes, and models\n- Supports cars, MPVs, trucks, motorcycles, buses, trailers, low speed vehicles and incomplete vehicles sold the U.S.A.\n- Find manufacturer model year VIN guides\n- Supports vehicles made since 1981\n- Comprehensive support for the vPIC API\n- Get results as Python objects (see vpic.TypedClient)\n- Get results as JSON (see vpic.Client)\n\n## Author\n\n- [@davidpeckham](https://github.com/davidpeckham)\n\n## Installation\n\n```bash \n  poetry add vpic-api\n```\n\n## vpic.TypedClient Examples\n\nvpic.TypedClient returns results as Python objects.\n\n### Decode a VIN\n\nGet the characteristics of a vehicle by it\'s 17-digit Vehicle Identification Number (VIN):\n\n```python\nfrom vpic import TypedClient\n\nc = TypedClient()\n\nresult = c.decode_vin("1FTMW1T88MFA00001")\n\nVehicle(\n    abs="",\n    ...\n    body_cab_type="Crew/ Super Crew/ Crew Max",\n    body_class="Pickup",\n    brake_system_desc="",\n    brake_system_type="Hydraulic",\n    ...\n    displacement_cc="3500.0",\n    displacement_ci="213.58310433156",\n    displacement_l="3.5",\n    ...\n    drive_type="4WD/4-Wheel Drive/4x4",\n    ...\n    engine_configuration="V-Shaped",\n    engine_cycles="",\n    engine_cylinders="6",\n    engine_hp="375",\n    engine_hp_to="",\n    engine_kw="279.6375",\n    engine_manufacturer="Ford",\n    engine_model="GTDI",\n    entertainment_system="",\n    error_code="0",\n    error_text="0 - VIN decoded clean. Check Digit (9th position) is correct",\n    ...\n    make="FORD",\n    make_id="460",\n    manufacturer="FORD MOTOR COMPANY, USA",\n    manufacturer_id="976",\n    model="F-150",\n    model_id="1801",\n    model_year="2021",\n    motorcycle_chassis_type="Not Applicable",\n    motorcycle_suspension_type="Not Applicable",\n    ...\n    plant_city="DEARBORN",\n    plant_company_name="",\n    plant_country="UNITED STATES (USA)",\n    plant_state="MICHIGAN",\n    ...\n    series="F-Series",\n    series2="",\n    ...\n    trim="SuperCrew-SSV",\n    ...\n    vin="1FTMW1T88MFA00001",\n    ...\n    vehicle_type="TRUCK ",\n    ...\n)\n```\n\n### Get the Models for a Make and Model Year\n\n```python\nget_models_for_make("TESLA", model_year=2020)\n\n[\n    Model(\n        model_id=1685,\n        model="Model S",\n        make_id=441,\n        make="TESLA",\n        vehicle_type_id=None\n    ),\n    Model(\n        model_id=10199,\n        model="Model X",\n        make_id=441,\n        make="TESLA",\n        vehicle_type_id=None\n    ),\n    Model(\n        model_id=17834,\n        model="Model 3",\n        make_id=441,\n        make="TESLA",\n        vehicle_type_id=None\n    ),\n    Model(\n        model_id=27027,\n        model="Model Y",\n        make_id=441,\n        make="TESLA",\n        vehicle_type_id=None\n        )\n]\n```\n\n## vpic.Client Examples\n\nvpic.Client returns JSON results from the vPIC API.\n\nThis client automatically standardizes variable names where vPIC uses inconsistent naming. Disable this to see the unaltered JSON responses:\n\n```python\nfrom vpic import Client\n\nc = Client(standardize_names=False)\n```\n\n### Decode a VIN\n\nGet the characteristics of a vehicle by it\'s 17-digit Vehicle Identification Number (VIN):\n\n```python\nfrom vpic import Client\n\nc = Client()\n\nresult = c.decode_vin("1FA6P8TD5M5100001", 2021)\n```\n\nHere are a few of the 130+ attributes vPIC returns for the VIN:\n\n```json\n{\n    "Doors": "2",\n    "ErrorCode": "0",\n    "ErrorText": "0 - VIN decoded clean. Check Digit (9th position) is correct",\n    "Make": "FORD",\n    "MakeId": "460",\n    "Manufacturer": "FORD MOTOR COMPANY, USA",\n    "ManufacturerId": "976",\n    "Model": "Mustang",\n    "ModelId": "1781",\n    "ModelYear": "2021",\n    "PlantCity": "FLATROCK",\n    "PlantCountry": "UNITED STATES (USA)",\n    "PlantState": "MICHIGAN",\n    "Series": "I4 Coupe",\n    "VIN": "1FA6P8TD5M5100001",\n    "VehicleType": "PASSENGER CAR",\n}\n```\n\n### Get the Models for a Make and Model Year\n\n```python\nresult = c.get_models_for_make("TESLA", 2021)\n```\n\n```json\n[\n    {\n        "MakeId": 441,\n        "Make": "TESLA",\n        "ModelId": 1685,\n        "Model": "Model S"\n    },\n    {\n        "MakeId": 441,\n        "Make": "TESLA",\n        "ModelId": 10199,\n        "Model": "Model X"\n    },\n    {\n        "MakeId": 441,\n        "Make": "TESLA",\n        "ModelId": 17834,\n        "Model": "Model 3"\n    },\n    {\n        "MakeId": 441,\n        "Make": "TESLA",\n        "ModelId": 27027,\n        "Model": "Model Y"\n    }\n]\n```\n',
    'author': 'David Peckham',
    'author_email': 'dave.peckham@icloud.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/davidpeckham/vpic-api',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from vpic.models import Vehicle, WMI
from vpic.typed_client import TypedClient


class TestDecode:
    def test_decode_vin(self, typed_client: TypedClient, responses):
        vehicle: Vehicle = typed_client.decode_vin("1FTMW1T88MFA00001")
        assert vehicle.make == "FORD"
        assert vehicle.model == "F-150"
        assert vehicle.trim == "SuperCrew-SSV"
        assert vehicle.vehicle_type == "TRUCK "

    def test_decode_vin_with_model_year(self, typed_client: TypedClient, responses):
        vehicle = typed_client.decode_vin("1FTMW1T88MFA00001", 2021)
        assert vehicle.make == "FORD"
        assert vehicle.model == "F-150"
        assert vehicle.trim == "SuperCrew-SSV"
        assert vehicle.vehicle_type == "TRUCK "

    def test_decode_vin_extended(self, typed_client: TypedClient, responses):
        vehicle = typed_client.decode_vin("1FTMW1T88MFA00001", 2021, extend=True)
        assert vehicle.make == "FORD"
        assert vehicle.model == "F-150"
        assert vehicle.trim == "SuperCrew-SSV"
        assert vehicle.vehicle_type == "TRUCK "

    def test_decode_partial_vin(self, typed_client: TypedClient, responses):
        vehicle = typed_client.decode_vin("5UXWX7C5*BA", 2011)
        assert vehicle.make == "BMW"
        assert vehicle.model == "X3"
        assert vehicle.trim == "xDrive35i"
        assert vehicle.vehicle_type == "MULTIPURPOSE PASSENGER VEHICLE (MPV)"

    def test_decode_vin_batch(self, typed_client: TypedClient, responses):
        vehicles = typed_client.decode_vin_batch(["5UXWX7C5*BA,2011", "5YJSA3DS*EF"])

        bmw = vehicles[0]
        assert bmw.model_year == "2011"
        assert bmw.make == "BMW"
        assert bmw.model == "X3"
        assert bmw.trim == "xDrive35i"
        assert bmw.vehicle_type == "MULTIPURPOSE PASSENGER VEHICLE (MPV)"

        tesla = vehicles[1]
        assert tesla.model_year == "2014"
        assert tesla.make == "TESLA"
        assert tesla.model == "Model S"
        assert tesla.trim == "w/DC Fast Charge"
        assert tesla.vehicle_type == "PASSENGER CAR"

    def test_decode_wmi(self, typed_client: TypedClient, responses):
        wmi: WMI = typed_client.decode_wmi("1FT")
        assert wmi.common_name == "Ford"
        assert wmi.created_on == "2015-03-23"
        assert wmi.date_available_to_public == "2015-01-01"
        assert wmi.make == "FORD"
        assert wmi.manufacturer == "FORD MOTOR COMPANY, USA"
        assert wmi.parent_company_name == ""
        assert wmi.updated_on is None
        assert wmi.url == "http://www.ford.com/"
        assert wmi.vehicle_type == "Truck "



import json
from pathlib import Path

from requests.adapters import HTTPAdapter
from vpic.session import VpicAPISession
from vpic.transforms import standardize


"""
Capture API responses for unit tests

Supports GET requests, but does not yet support POST requests
like decode_vin_batch.

"""

session = VpicAPISession()
session.mount(
    "https://vpic.nhtsa.dot.gov/api/vehicles",
    HTTPAdapter(pool_connections=2, max_retries=5),
)

response_files = list(Path("tests").glob("**/*.json"))
for rf in response_files:
    with rf.open(encoding="utf-8") as fp:
        data = json.load(fp)
        url = data["url"]
        limit = data.get("limit", None)
        post_data = data.get("post_data", None)

        if post_data is None:
            resp = session.get(url)
        else:
            resp = session.post(url=url, data=post_data)

        print(f"{fp.name} {url} {resp.status_code}")

        if resp.status_code < 400:
            raw_response = resp.json()
            results = raw_response["Results"]
            if limit == 1:
                transformed_response = standardize(results[0])
            else:
                transformed_response = standardize(results)
        else:
            raw_response = None
            transformed_response = None

    if raw_response is None:
        continue

    with rf.open(encoding="utf-8", mode="w") as fp:
        data["content"] = raw_response
        data["expected_result"] = transformed_response
        json.dump(data, fp, indent=4)



from . import expected_result


class TestModels:
    def test_get_models_for_makeid(self, vpic, responses, datadir):
        assert vpic.get_models_for_make(441) == expected_result(
            datadir / "get-models-for-makeid.json"
        )

    def test_get_models_for_makeid_year(self, vpic, responses, datadir):
        assert vpic.get_models_for_make(441, model_year=2020) == expected_result(
            datadir / "get-models-for-makeid-year.json"
        )

    def test_get_models_for_makeid_vehicletype(self, vpic, responses, datadir):
        assert vpic.get_models_for_make(441, vehicle_type="Car") == expected_result(
            datadir / "get-models-for-makeid-vehicletype.json"
        )

    def test_get_models_for_makeid_year_vehicletype(self, vpic, responses, datadir):
        assert vpic.get_models_for_make(
            441, model_year=2021, vehicle_type="Car"
        ) == expected_result(datadir / "get-models-for-makeid-year-vehicletype.json")

    def test_get_models_for_make(self, vpic, responses, datadir):
        assert vpic.get_models_for_make("TESLA") == expected_result(
            datadir / "get-models-for-make.json"
        )

    def test_get_models_for_make_year(self, vpic, responses, datadir):
        assert vpic.get_models_for_make("TESLA", model_year=2020) == expected_result(
            datadir / "get-models-for-make-year.json"
        )

    def test_get_models_for_make_vehicletype(self, vpic, responses, datadir):
        assert vpic.get_models_for_make("TESLA", vehicle_type="Car") == expected_result(
            datadir / "get-models-for-make-vehicletype.json"
        )

    def test_get_models_for_make_year_vehicletype(self, vpic, responses, datadir):
        assert vpic.get_models_for_make(
            "TESLA", model_year=2020, vehicle_type="Car"
        ) == expected_result(datadir / "get-models-for-make-year-vehicletype.json")

    def test_get_canadian_vehicle_specifications(self, vpic, responses, datadir):
        assert vpic.get_canadian_vehicle_specifications(
            2011, make="Acura", model="", units=""
        ) == expected_result(datadir / "get-canadian-vehicle-specifications.json")
